import os

DEFAULT_DB_PATH = os.path.expanduser("~/.local/share/tesseract/tesseract.db")
DEFAULT_INDEX_PATH = os.path.expanduser("~/tesseract")
DEFAULT_API_URL = "http://localhost:8000"
CONFIG_PATH = os.path.expanduser("~/.config/tesseract/config.ini")
